//===========================================================================
//
//  Code written for Seeeduino v4.2 runnung at 3.3v
//
//  CRYSTALFONTZ CFAL12832DB 128x23 OLED in SPI mode
//
//  ref: https://www.crystalfontz.com/product/cfal12832db
//
//  2018 - 02 - 24 Brent A. Crosby
//===========================================================================
//This is free and unencumbered software released into the public domain.
//
//Anyone is free to copy, modify, publish, use, compile, sell, or
//distribute this software, either in source code form or as a compiled
//binary, for any purpose, commercial or non-commercial, and by any
//means.
//
//In jurisdictions that recognize copyright laws, the author or authors
//of this software dedicate any and all copyright interest in the
//software to the public domain. We make this dedication for the benefit
//of the public at large and to the detriment of our heirs and
//successors. We intend this dedication to be an overt act of
//relinquishment in perpetuity of all present and future rights to this
//software under copyright law.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
//OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
//ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//OTHER DEALINGS IN THE SOFTWARE.
//
//For more information, please refer to <http://unlicense.org/>
//===========================================================================
//
// Display is Crystalfontz CFAL12832D-B
//   https://www.crystalfontz.com/product/cfal12832db
//
// Breakout board and wires may be useful:
//   https://www.crystalfontz.com/product/cfa10054
//   https://www.crystalfontz.com/product/wrjmpy41
//   https://www.crystalfontz.com/product/wrjmpy40
//
// The controller is a Solomon Systech SSD1306
//   https://www.crystalfontz.com/controllers/SolomonSystech/SSD1306/
//
//============================================================================
#include <avr/io.h>
#include <SPI.h>
// C:\Program Files (x86)\Arduino\hardware\arduino\avr\libraries\SPI\src\SPI.cpp
// C:\Program Files (x86)\Arduino\hardware\arduino\avr\libraries\SPI\src\SPI.h
#include <avr/pgmspace.h>
//============================================================================
//
// The CFAL12832D-B is a 3.3v device. You need a 3.3v Arduino or the
// Seeeduino v4.2 set to 3.3v to operate this code properly:
// http://www.seeedstudio.com/item_detail.html?p_id=2517
//
//============================================================================
// LCD SPI & control lines
//   ARD   | Port | OLED                     | Wire
//---------+------+--------------------------+------------
//  3.3V   |      | 3.3v VDD and VBAT        | Orange
//  GND    |      | Ground                   | Brown
// --------+------+--------------------------+------------
//  #8/D8  |  PB0 | LCD_RS                   | Yellow
//  #9/D9  |  PB1 | LCD_RESET                | Purple
// #10/D10 |  PB2 | LCD_CS_NOT (or SPI SS)   | Grey
// #11/D11 |  PB3 | LCD_MOSI (hardware SPI)  | Green 
// #12/D12 |  PB4 | not used (would be MISO) | Blue
// #13/D13 |  PB5 | LCD_SCK (hardware SPI)   | White
//============================================================================
#define CLR_RS    (PORTB &= ~(0x01))
#define SET_RS    (PORTB |=  (0x01))
#define CLR_RST   (PORTB &= ~(0x02))
#define SET_RST   (PORTB |=  (0x02))
#define CLR_CS    (PORTB &= ~(0x04))
#define SET_CS    (PORTB |=  (0x04))
#define CLR_MOSI  (PORTB &= ~(0x08))
#define SET_MOSI  (PORTB |=  (0x08))
#define CLR_SCK   (PORTB &= ~(0x20))
#define SET_SCK   (PORTB |=  (0x20))
//============================================================================
//Sample screens saved in flash
typedef struct
  {
  uint8_t
    bitmap_data[4][128];
  } SCREEN_IMAGE;

const SCREEN_IMAGE Logo_1_I2C PROGMEM =
  {
    {
      {0x00,0x00,0x00,0xC0,0xE0,0xF0,0xF8,0xFC,0x1E,0x0F,0x07,0x03,0x07,0xCF,0xE6,0xF0,0xF8,0xFC,0x1E,0x0F,0x07,0x03,0x07,0x0F,0xFE,0xFC,0xF8,0xF0,0xE0,0x00,0x00,0x00,0x00,0x00,0x40,0x40,0x7F,0x40,0x40,0x0E,0x15,0x15,0x15,0x08,0x00,0x1B,0x04,0x04,0x1B,0x00,0x10,0x3E,0x11,0x12,0x00,0x00,0xE0,0xF8,0xFC,0xFE,0x0E,0x03,0x03,0x01,0x01,0x01,0x03,0x03,0x0E,0xFE,0xFC,0xF8,0xE0,0x00,0x00,0x00,0x01,0x01,0xFF,0xFF,0xFF,0xFF,0x01,0x01,0x01,0x01,0x01,0x03,0x03,0x0F,0x38,0x00,0x00,0x00,0x01,0x01,0xFF,0xFF,0xFF,0xFF,0x01,0x01,0x81,0xF1,0x01,0x01,0x03,0x07,0x1C,0x00,0x00,0x00,0x01,0x01,0xFF,0xFF,0xFF,0xFF,0x01,0x01,0x01,0x03,0x03,0x0E,0xFE,0xFC,0xF8,0xE0},
      {0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0x00,0xFF,0xFF,0xFF,0xFF,0xFF,0x00,0xB6,0xB6,0xB6,0xB6,0xB6,0xB6,0xB6,0xB6,0xB6,0xB6,0xB6,0xB6,0xB6,0xB6,0x00,0x00,0x00,0x1E,0x29,0x49,0x49,0x06,0x00,0x1B,0x04,0x04,0x1B,0x00,0x36,0x49,0x49,0x49,0x36,0x00,0x00,0x00,0x00,0x0F,0x3F,0x7F,0xFF,0xE0,0x80,0x80,0x00,0x00,0x00,0x80,0x80,0xE0,0xFF,0x7F,0x3F,0x0F,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF,0xFF,0xFF,0x01,0x01,0x03,0x1F,0x00,0x80,0xC0,0xF0,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x80,0xC0,0xF0,0xFF,0x7F,0x3F,0x0F},
      {0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x00,0xFF,0xFF,0xFF,0xFF,0xFF,0x00,0x6D,0x6D,0x6D,0x6D,0x6D,0x6D,0x6D,0x6D,0x6D,0x6D,0x6D,0x6D,0x6D,0x6D,0x00,0x0C,0x0C,0xFC,0xFC,0x0C,0x0C,0x00,0x60,0xE0,0xE0,0xE0,0x60,0x60,0x60,0x60,0x00,0xC0,0xF0,0x38,0x1C,0x0C,0x0C,0x0C,0x1C,0x38,0x30,0x00,0x01,0x01,0x01,0x01,0x0D,0x0D,0xFD,0xFC,0x0C,0x0C,0x00,0x00,0x0C,0x1C,0x3C,0x7D,0xED,0xCD,0x8D,0x0D,0x0D,0x0D,0x01,0xF0,0xF8,0x9C,0x0C,0x0C,0x0C,0x0C,0x0C,0x9C,0xF8,0xF1,0x01,0x0D,0x1D,0xB9,0xF1,0xE1,0xF1,0xB9,0x1D,0x0D,0x01,0x31,0x39,0x1C,0x0C,0x0C,0x0C,0x0D,0x0D,0x9D,0xF9,0x71,0x01,0x0D,0x1D,0x3D,0x7D,0xED,0xCC,0x8C,0x0C,0x0C,0x0C},
      {0x00,0x00,0x00,0x03,0x07,0x0F,0x1F,0x3F,0x78,0xF0,0xE0,0xC0,0xE0,0xF3,0x67,0x0F,0x1F,0x3F,0x78,0xF0,0xE0,0xC0,0xE0,0xF0,0x7F,0x3F,0x1F,0x0F,0x07,0x00,0x00,0x00,0x00,0x00,0xC0,0xC0,0xFF,0xFF,0xC0,0xC0,0x00,0x38,0x78,0xE1,0xC3,0xC7,0xEE,0x7C,0x38,0x00,0x0F,0x3F,0x70,0xE0,0xC0,0xC0,0xC0,0xE0,0x70,0x30,0x00,0x00,0x00,0x00,0x00,0x30,0x70,0xFF,0xFF,0x00,0x00,0x00,0x00,0x38,0x78,0xE0,0xC0,0xC0,0xC1,0xC3,0xE7,0x7E,0x3C,0x00,0x00,0x3D,0x7F,0xE7,0xC3,0xC3,0xC3,0xE7,0x7F,0x3D,0x00,0x00,0x06,0x07,0x03,0x01,0x00,0x01,0x03,0x07,0x06,0x00,0x30,0x70,0xE0,0xC3,0xC3,0xC3,0xC3,0xC3,0xE7,0x7E,0x3C,0x00,0x38,0x78,0xE0,0xC0,0xC0,0xC1,0xC3,0xE7,0x7E,0x3C}
    }
  };

const SCREEN_IMAGE Logo_2_I2C PROGMEM =
  {
    {
      {0x00,0x00,0x00,0x80,0xC0,0xE0,0xF0,0xF8,0x0C,0x06,0x03,0x01,0x03,0x06,0xCC,0xE0,0xF0,0xF8,0xFC,0x06,0x03,0x01,0x03,0x06,0x0C,0xF8,0xF0,0xE0,0xC0,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0xFF,0xFF,0x01,0x00,0x00,0x00,0xC3,0xC7,0x0D,0x19,0xF3,0xE3,0x00,0x00,0x7C,0xFE,0x83,0x01,0x01,0xC7,0xC6,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF8,0xFE,0xFE,0x83,0x01,0x01,0x01,0x03,0x06,0x0E,0x3C,0xF8,0xF1,0xC1,0x03,0x1F,0xFF,0xFF,0xE1,0x01,0x01,0x01,0x03,0x03,0x07,0x06,0x08,0x01,0x01,0x03,0x1F,0xFF,0xFF,0xE1,0x01,0x01,0x01,0x01,0xE3,0x03,0x06,0x08,0x01,0x01,0x03,0x1F,0xFF,0xFF,0xE1,0x01,0x01,0x03,0x02,0x06,0x0E,0x7C,0xF8,0xE0,0xC0},
      {0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0x00,0x00,0x00,0xFF,0xFF,0xFF,0xFF,0xFF,0x00,0xB6,0xB6,0xB6,0xB6,0xB6,0xB6,0xB6,0xB6,0xB6,0xB6,0xB6,0xB6,0xB6,0x00,0x00,0x00,0x84,0xFC,0x04,0x00,0x8D,0x15,0x25,0x25,0xC4,0x00,0xD8,0x24,0x25,0x25,0xD9,0x01,0x6C,0x10,0x10,0x6C,0x00,0x89,0x25,0x25,0x25,0xD8,0x00,0x8C,0x14,0x24,0x24,0xC4,0x00,0x00,0x07,0x0F,0x1F,0x3C,0x70,0x60,0xC0,0x80,0x80,0x80,0xC1,0x7F,0x3F,0x1F,0x00,0x00,0x87,0xFF,0xFF,0xF8,0xC0,0x80,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x87,0xFF,0xFF,0xF9,0xC1,0x81,0x83,0x87,0x88,0xC0,0xF0,0xC0,0x00,0x00,0x80,0x87,0xFF,0xFF,0xF8,0xC0,0x80,0x80,0xC0,0x60,0x7F,0x3F,0x0F},
      {0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x00,0x00,0x00,0xFF,0xFF,0xFF,0xFF,0xFF,0x00,0x6D,0x6D,0x6D,0x6D,0x6D,0x6D,0x6D,0x6D,0x6D,0x6D,0x6D,0x6D,0x6D,0x00,0xC0,0xF0,0x38,0x1D,0x0C,0x0C,0x0C,0x1D,0x39,0x31,0x00,0xFC,0xFC,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x81,0xE7,0x7E,0x78,0xE1,0x81,0x01,0x00,0x88,0xCC,0xCD,0xCD,0xFD,0x78,0x00,0x00,0x00,0xF8,0xFC,0x0C,0x0C,0x0C,0x00,0x38,0x7C,0xCC,0xCC,0xCC,0xCC,0xFC,0xFC,0x00,0x0C,0x0C,0xFC,0xFC,0x0C,0x0C,0x00,0xFC,0xFC,0x00,0x00,0x00,0xF0,0xF8,0x1C,0x0C,0x0C,0x1C,0xF8,0xF0,0x00,0xFC,0xFC,0x00,0x00,0x00,0xFC,0xFC,0x00,0x00,0xF8,0xFC,0x0C,0x0C,0x0C,0x00,0x1C,0x3C,0x6C,0xCC,0x8C,0x0C},
      {0x00,0x00,0x00,0x01,0x03,0x07,0x0F,0x1F,0x30,0x60,0xC0,0x80,0xC0,0x60,0x33,0x07,0x0F,0x1F,0x3F,0x60,0xC0,0x80,0xC0,0x60,0x30,0x1F,0x0F,0x07,0x03,0x01,0x00,0x00,0x00,0x00,0x0F,0x3F,0x70,0xE0,0xC0,0xC0,0xC0,0xE0,0x70,0x30,0x00,0x07,0x07,0x03,0x06,0x06,0x07,0x03,0x00,0x06,0x07,0x01,0x00,0x00,0x01,0x07,0x06,0x00,0x03,0x07,0x06,0x06,0x06,0x02,0x00,0x06,0x06,0x7F,0x7F,0x06,0x06,0x00,0x00,0x00,0x02,0x06,0x06,0x06,0x06,0x07,0x03,0x00,0x00,0xC0,0xFF,0xFF,0x00,0x03,0x03,0x3F,0x7F,0xE3,0xC3,0xC0,0x01,0x03,0x07,0x06,0x06,0x07,0x03,0x01,0x00,0x07,0x07,0x06,0x06,0x07,0x03,0x01,0x06,0x06,0x7F,0x7F,0x06,0x06,0x00,0x00,0x06,0x06,0x06,0x06,0x07,0x07}
    }
  };

const SCREEN_IMAGE odd_checker PROGMEM =
  {
    {
      {0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA},
      {0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA},
      {0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA},
      {0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA}
    }
  };

const SCREEN_IMAGE even_checker PROGMEM =
  {
    {
      {0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55},
      {0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55},
      {0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55},
      {0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55,0xAA,0x55}
    }
  };

const SCREEN_IMAGE all_on PROGMEM =
  {
    {
      {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
      {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
      {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF},
      {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF}
    }
  };

const SCREEN_IMAGE even_h_lines PROGMEM =
  {
    {
      {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
      {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
      {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA},
      {0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA}
    }
  };

const SCREEN_IMAGE odd_h_lines PROGMEM =
  {
    {
      {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55},
      {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55},
      {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55},
      {0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55}
    }
  };

const SCREEN_IMAGE even_v_lines PROGMEM =
  {
    {
      {0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00},
      {0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00},
      {0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00},
      {0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00}
    }
  };

const SCREEN_IMAGE odd_v_lines PROGMEM =
  {
    {
      {0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF},
      {0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF},
      {0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF},
      {0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF,0x00,0xFF}
    }
  };

const SCREEN_IMAGE text_8x8 PROGMEM =
  {
    {
      {0x00,0x00,0x82,0xFE,0xFE,0x02,0x00,0x00,0x00,0x00,0x22,0xBE,0xBE,0x02,0x00,0x00,0x00,0x3E,0x3E,0x20,0x20,0x3E,0x1E,0x00,0x00,0x1C,0x3E,0x2A,0x2A,0x3A,0x18,0x00,0x00,0x00,0x05,0x07,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1C,0x3E,0x22,0x22,0x3E,0x1C,0x00,0x00,0x3E,0x3E,0x20,0x20,0x3E,0x1E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x38,0x68,0xC8,0xFE,0xFE,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x82,0xFE,0xFE,0x02,0x00,0x00,0x00,0x00,0x22,0xBE,0xBE,0x02,0x00,0x00,0x00,0x3E,0x3E,0x20,0x20,0x3E,0x1E,0x00,0x00,0x1C,0x3E,0x2A,0x2A,0x3A,0x18,0x00,0x00,0x12,0x3A,0x2A,0x2A,0x2E,0x24,0x00},
      {0x00,0x1C,0x3E,0x22,0x22,0x36,0x14,0x00,0x00,0x82,0xFE,0xFE,0x10,0x20,0x3E,0x1E,0x00,0x04,0x2E,0x2A,0x2A,0x3C,0x1E,0x02,0x00,0x22,0x3E,0x1E,0x22,0x38,0x18,0x00,0x00,0x04,0x2E,0x2A,0x2A,0x3C,0x1E,0x02,0x00,0x1C,0x3E,0x22,0x22,0x36,0x14,0x00,0x00,0x00,0x20,0x7C,0xFE,0x22,0x24,0x00,0x00,0x1C,0x3E,0x2A,0x2A,0x3A,0x18,0x00,0x00,0x22,0x3E,0x1E,0x22,0x38,0x18,0x00,0x00,0x12,0x3A,0x2A,0x2A,0x2E,0x24,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1C,0x3E,0x2A,0x2A,0x3A,0x18,0x00,0x00,0x04,0x2E,0x2A,0x2A,0x3C,0x1E,0x02,0x00,0x1C,0x3E,0x22,0x22,0x36,0x14,0x00,0x00,0x82,0xFE,0xFE,0x10,0x20,0x3E,0x1E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
      {0x00,0x39,0x3D,0x05,0x05,0x3F,0x3E,0x00,0x00,0x1C,0x3E,0x22,0x22,0x3E,0x1C,0x00,0x00,0x3C,0x3E,0x02,0x02,0x3C,0x3E,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1C,0x3E,0x22,0x22,0x36,0x14,0x00,0x00,0x04,0x2E,0x2A,0x2A,0x3C,0x1E,0x02,0x00,0x3E,0x3E,0x20,0x20,0x3E,0x1E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x12,0x3A,0x2A,0x2A,0x2E,0x24,0x00,0x00,0x82,0xFE,0xFE,0x10,0x20,0x3E,0x1E,0x00,0x1C,0x3E,0x22,0x22,0x3E,0x1C,0x00,0x00,0x3C,0x3E,0x06,0x0C,0x06,0x3E,0x3C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x42,0x42,0xFE,0xFE,0x02,0x02,0x00,0x00,0x3C,0x7E,0xD2,0x92,0x9E,0x0C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
      {0x00,0xFE,0xFE,0x02,0x02,0xFE,0xFE,0x00,0x00,0x12,0x3A,0x2A,0x2A,0x2E,0x24,0x00,0x00,0x00,0x22,0xBE,0xBE,0x02,0x00,0x00,0x00,0x3E,0x3E,0x20,0x20,0x3E,0x1E,0x00,0x00,0x19,0x3D,0x25,0x25,0x1F,0x3E,0x20,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x2E,0x2A,0x2A,0x3C,0x1E,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x6C,0xFE,0x92,0x92,0xFE,0x6C,0x00,0x00,0x22,0x36,0x1C,0x08,0x1C,0x36,0x22,0x00,0x6C,0xFE,0x92,0x92,0xFE,0x6C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x12,0x7E,0xFE,0x92,0xC0,0x40,0x00,0x00,0x1C,0x3E,0x22,0x22,0x3E,0x1C,0x00,0x00,0x3E,0x3E,0x20,0x20,0x3E,0x1E,0x00,0x00,0x00,0x20,0x7C,0xFE,0x22,0x24,0x00}
    }
  };

const SCREEN_IMAGE text_6x8 PROGMEM =
  {
    {
      {0x00,0x00,0x00,0xFE,0x02,0x00,0x00,0x00,0x00,0xBE,0x02,0x00,0x00,0x3E,0x20,0x20,0x1E,0x00,0x00,0x1C,0x2A,0x2A,0x2A,0x10,0x00,0x00,0x07,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3C,0x06,0x0C,0x06,0x3C,0x00,0x00,0x00,0xBE,0x02,0x00,0x00,0x20,0x7C,0x22,0x24,0x00,0x00,0xFE,0x20,0x20,0x1E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x28,0x48,0xFE,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFE,0x02,0x00,0x00,0x00,0x00,0xBE,0x02,0x00,0x00,0x3E,0x20,0x20,0x1E,0x00,0x00,0x1C,0x2A,0x2A,0x2A,0x10,0x00,0x10,0x2A,0x2A,0x2A,0x04,0x00,0x00,0x06,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
      {0x00,0x46,0x8A,0x92,0x92,0x62,0x00,0x00,0x42,0xFE,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1C,0x22,0x22,0x22,0x14,0x00,0xFE,0x20,0x20,0x1E,0x00,0x00,0x04,0x2A,0x2A,0x2A,0x1E,0x00,0x22,0x1E,0x22,0x20,0x10,0x00,0x04,0x2A,0x2A,0x2A,0x1E,0x00,0x1C,0x22,0x22,0x22,0x14,0x00,0x20,0x7C,0x22,0x24,0x00,0x00,0x1C,0x2A,0x2A,0x2A,0x10,0x00,0x22,0x1E,0x22,0x20,0x10,0x00,0x10,0x2A,0x2A,0x2A,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1C,0x22,0x22,0x22,0x1C,0x00,0x3E,0x20,0x20,0x1E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1C,0x2A,0x2A,0x2A,0x10,0x00,0x04,0x2A,0x2A,0x2A,0x1E,0x00,0x1C,0x22,0x22,0x22,0x14,0x00,0xFE,0x20,0x20,0x1E,0x00,0x00,0x00},
      {0x00,0x00,0x00,0xBE,0x02,0x00,0x00,0x10,0x2A,0x2A,0x2A,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x38,0x04,0x02,0x04,0x38,0x00,0x1C,0x2A,0x2A,0x2A,0x10,0x00,0x22,0x1E,0x22,0x20,0x10,0x00,0x39,0x05,0x06,0x3C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x2A,0x2A,0x2A,0x04,0x00,0x3E,0x20,0x18,0x20,0x1E,0x00,0x04,0x2A,0x2A,0x2A,0x1E,0x00,0x00,0x00,0xFE,0x02,0x00,0x00,0x00,0x00,0xFE,0x02,0x00,0x00,0x00,0x82,0x7C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x18,0x25,0x25,0x25,0x3E,0x00,0x1C,0x2A,0x2A,0x2A,0x10,0x00,0x20,0x7C,0x22,0x24,0x00,0x00,0x10,0x2A,0x2A,0x2A,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},
      {0x00,0xFC,0x02,0x02,0x02,0xFC,0x00,0x10,0x2A,0x2A,0x2A,0x04,0x00,0x00,0x00,0xBE,0x02,0x00,0x00,0x3E,0x20,0x20,0x1E,0x00,0x00,0x18,0x25,0x25,0x25,0x3E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x2A,0x2A,0x2A,0x1E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3C,0x52,0x92,0x92,0x0C,0x00,0x36,0x08,0x08,0x36,0x00,0x00,0x6C,0x92,0x92,0x92,0x6C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x7E,0x90,0x90,0x00,0x00,0x1C,0x22,0x22,0x22,0x1C,0x00,0x3E,0x20,0x20,0x1E,0x00,0x00,0x20,0x7C,0x22,0x24,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7C,0x82,0x00,0x00,0x00,0x00,0x00,0xBE,0x02,0x00,0x00,0x20,0x7C,0x22,0x24,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}
    }
  };

const SCREEN_IMAGE text_4x6 PROGMEM =
  {
    {
      {0x02,0x8E,0x88,0x00,0x84,0x8A,0x84,0x00,0x84,0x0A,0x0A,0x00,0xC0,0xAE,0x00,0x00,0x04,0x8A,0x04,0x00,0x82,0x8E,0x08,0x00,0x04,0x0A,0x0F,0x00,0x8E,0x82,0x8E,0x00,0x80,0x00,0x80,0x00,0x00,0xAE,0x80,0x00,0x0F,0x8A,0x84,0x00,0x02,0x0E,0x08,0x00,0x8E,0x82,0x0E,0x00,0x8E,0x8C,0x8E,0x00,0x80,0x82,0x00,0x00,0xC0,0x80,0x00,0x00,0x1C,0xA2,0xA2,0x00,0x8E,0x02,0x8E,0x00,0x0E,0x88,0x80,0x00,0x06,0x8A,0x0E,0x00,0xBE,0x8A,0x04,0x00,0x80,0xAE,0x00,0x00,0x08,0x9E,0x8A,0x00,0xCE,0x82,0x0E,0x00,0x0E,0x08,0x00,0x00,0x00,0x80,0x80,0x00,0x44,0x4A,0xCA,0x00,0x0E,0x88,0x80,0x00,0x06,0x8A,0x8E,0x00,0x02,0x0E,0x08,0x00,0xC3,0x82,0x00,0x00,0x00,0x80,0x80,0x00},
      {0xC2,0x27,0xC2,0x00,0xC3,0xE0,0xA3,0x00,0xE3,0x82,0x60,0x00,0x03,0xE2,0x01,0x00,0x40,0xAB,0xA0,0x00,0xE0,0x23,0xE2,0x00,0x00,0xE0,0x00,0x00,0x61,0xA2,0xE3,0x00,0x03,0x03,0x03,0x00,0x83,0xE3,0x82,0x00,0xE2,0x87,0x02,0x00,0x00,0xE0,0x00,0x00,0xE0,0x83,0x62,0x00,0xD3,0x90,0xF3,0x00,0x00,0xE3,0x02,0x00,0x03,0xE2,0x01,0x00,0x03,0xE3,0x02,0x00,0x63,0xA2,0xE1,0x00,0x01,0x02,0x0F,0x00,0x40,0xAB,0xF0,0x00,0xE0,0x23,0xE2,0x00,0x00,0xE3,0x02,0x00,0x23,0xE3,0x82,0x00,0x00,0x00,0x00,0x00,0x40,0xA0,0x40,0x00,0x43,0xA3,0xE2,0x00,0x03,0xE2,0x03,0x00,0x43,0xA3,0x42,0x00,0x32,0x27,0x02,0x00,0x00,0x00,0x00,0x00,0xE3,0xC2,0xE1,0x00,0x43,0xA3,0x42,0x00},
      {0xF0,0x50,0xA0,0x00,0xF0,0x10,0xF0,0x00,0x03,0xF0,0x00,0x00,0x00,0x02,0x00,0x00,0x10,0xF0,0x10,0x00,0x00,0xF0,0x00,0x00,0x00,0x03,0x00,0x00,0x10,0xF0,0x10,0x00,0xD0,0x50,0x60,0x00,0x00,0x01,0x02,0x00,0xF0,0x40,0x80,0x00,0xF0,0x42,0xB0,0x00,0xF0,0x50,0x10,0x00,0x00,0xF0,0x00,0x00,0x00,0xF2,0x00,0x00,0x80,0x73,0x80,0x00,0x00,0x03,0x00,0x00,0xF0,0x10,0xF0,0x00,0xD0,0x50,0x60,0x00,0xF0,0x50,0x10,0x00,0xF0,0x10,0x10,0x00,0xF0,0x52,0x10,0x00,0xD0,0x50,0x60,0x00,0xD0,0x50,0x60,0x00,0x00,0x00,0x00,0x00,0xF0,0x10,0x13,0x00,0x20,0x52,0x20,0x00,0x70,0x40,0x00,0x00,0x60,0x70,0x50,0x00,0x70,0x60,0x70,0x00,0x00,0x00,0x00,0x00,0x00,0x70,0x00,0x00},
      {0x69,0xA9,0xB0,0x00,0x01,0xB8,0x01,0x00,0x01,0xF9,0x01,0x00,0x00,0xF8,0x00,0x00,0x33,0x09,0x31,0x00,0x01,0x01,0x01,0x00,0x08,0x38,0x20,0x00,0x39,0x31,0x39,0x00,0x18,0x29,0x39,0x00,0x00,0xF8,0x00,0x00,0x01,0xF9,0x00,0x00,0x01,0x01,0x00,0x00,0x61,0x21,0xF9,0x00,0x29,0x11,0x29,0x00,0x71,0xA9,0x11,0x00,0x01,0x00,0x01,0x00,0x20,0x78,0xA0,0x00,0x11,0x28,0x11,0x00,0x38,0x21,0x19,0x00,0x21,0x79,0x29,0x00,0x01,0x28,0x00,0x00,0x01,0x01,0x01,0x00,0x88,0xA9,0x51,0x00,0x98,0xA9,0x49,0x00,0x28,0x10,0x28,0x00,0xE9,0xA8,0x90,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x00}
    }
  };
//----------------------------------------------------------------------------
#define NUMBER_OF_SCREENS (12)
const SCREEN_IMAGE *const screens[NUMBER_OF_SCREENS] PROGMEM=
  {
  &Logo_1_I2C,
  &Logo_2_I2C,
  &odd_checker,
  &even_checker,
  &all_on,
  &even_h_lines,
  &odd_h_lines,
  &even_v_lines,
  &odd_v_lines,
  &text_8x8,
  &text_6x8,
  &text_4x6
  };
//============================================================================
void sendcommand(uint8_t command)
  {
  // Select the LCD's command register
  CLR_RS;
  // Select the LCD controller
  CLR_CS;
  //Send the command via SPI:
  SPI.transfer(command);
  // Deselect the LCD controller
  SET_CS;
  }
//============================================================================
void senddata(uint8_t data)
  {
  // Select the LCD's data register
  SET_RS;
  // Select the LCD controller
  CLR_CS;
  //Send the command via SPI:
  SPI.transfer(data);
  // Deselect the LCD controller
  SET_CS;
  }
//============================================================================
// Straight up code: ~1.3mS to update
void show_128_x_4_bitmap(const SCREEN_IMAGE *OLED_image)
  {
  uint8_t
    column;
  uint8_t
    row;

  for(row=0;row<4;row++)
    {
    sendcommand(0x00);      //lower column address
    sendcommand(0x10);      //upper column address
    sendcommand(0xB0+row);  //set page address
    for(column=0;column<128;column++)
      {
      //Read this byte from the program memory / flash
      senddata(pgm_read_byte( &(OLED_image  ->bitmap_data[row][column]) ));
      }
    }
  }
//============================================================================
//With an 8MHz SPI clock, this function takes 617uS
//128*4*8 =4096 bits in 617uS = 6.638 megabits per second
void show_128_x_4_bitmap_faster(const SCREEN_IMAGE *OLED_image)
  {
  uint8_t
    column;
  uint8_t
    row;
  uint8_t
    this_byte;

  for(row=0;row<4;row++)
    {
    sendcommand(0x00);      //lower column address
    sendcommand(0x10);      //upper column address
    sendcommand(0xB0+row);  //set page address
    // Select the LCD's data register
    SET_RS;
    // Select the LCD controller
    CLR_CS;
    //Use a while loop, faster than a for.
    column=0;
    //Get the first flash access out of the way.
    this_byte=pgm_read_byte( &(OLED_image  ->bitmap_data[row][column]) );
    do
      {
      //Start shifting this byte
      SPDR=this_byte;
      //Do all the other things we can while this byte is shifting out the port.
      column++;
      this_byte=pgm_read_byte( &(OLED_image  ->bitmap_data[row][column]) );
      //Additional delay needed (7 nops works, 6 fails)
      __asm__("nop\n\t"); 
      __asm__("nop\n\t"); 
      __asm__("nop\n\t"); 
      __asm__("nop\n\t"); 
      __asm__("nop\n\t"); 
      __asm__("nop\n\t"); 
      __asm__("nop\n\t"); 
      }
    while(column<=127);
    //Wait for the last transmission to complete before
    //raising CS.
    while (!(SPSR & _BV(SPIF))) ; // wait
    // Deselect the LCD controller
    SET_CS;
    }
  }
//============================================================================
//Updated code, 2016-04-14
#define SSD1306B_DCDC_CONFIG_PREFIX_8D          (0x8D)
#define SSD1306B_DCDC_CONFIG_7p5v_14            (0x14)
#define SSD1306B_DCDC_CONFIG_6p0v_15            (0x15)
#define SSD1306B_DCDC_CONFIG_8p5v_94            (0x94)
#define SSD1306B_DCDC_CONFIG_9p0v_95            (0x95)
#define SSD1306B_DISPLAY_OFF_YES_SLEEP_AE       (0xAE)
#define SSD1306B_DISPLAY_ON_NO_SLEEP_AF         (0xAF)
#define SSD1306B_CLOCK_DIVIDE_PREFIX_D5         (0xD5)
#define SSD1306B_MULTIPLEX_RATIO_PREFIX_A8      (0xA8)
#define SSD1306B_DISPLAY_OFFSET_PREFIX_D3       (0xD3)
#define SSD1306B_DISPLAY_START_LINE_40          (0x40)
#define SSD1306B_SEG0_IS_COL_0_A0               (0xA0)
#define SSD1306B_SEG0_IS_COL_127_A1             (0xA1)
#define SSD1306B_SCAN_DIR_UP_C0                 (0xC0)
#define SSD1306B_SCAN_DIR_DOWN_C8               (0xC8)
#define SSD1306B_COM_CONFIG_PREFIX_DA           (0xDA)
#define SSD1306B_COM_CONFIG_SEQUENTIAL_LEFT_02  (0x02)
#define SSD1306B_COM_CONFIG_ALTERNATE_LEFT_12   (0x12)
#define SSD1306B_COM_CONFIG_SEQUENTIAL_RIGHT_22 (0x22)
#define SSD1306B_COM_CONFIG_ALTERNATE_RIGHT_32  (0x32)
#define SSD1306B_CONTRAST_PREFIX_81             (0x81)
#define SSD1306B_PRECHARGE_PERIOD_PREFIX_D9     (0xD9)
#define SSD1306B_VCOMH_DESELECT_PREFIX_DB       (0xDB)
#define SSD1306B_VCOMH_DESELECT_0p65xVCC_00     (0x00)
#define SSD1306B_VCOMH_DESELECT_0p71xVCC_10     (0x10)
#define SSD1306B_VCOMH_DESELECT_0p77xVCC_20     (0x20)
#define SSD1306B_VCOMH_DESELECT_0p83xVCC_30     (0x30)
#define SSD1306B_ENTIRE_DISPLAY_FORCE_ON_A5     (0xA5)
#define SSD1306B_ENTIRE_DISPLAY_NORMAL_A4       (0xA4)
#define SSD1306B_INVERSION_NORMAL_A6            (0xA6)
#define SSD1306B_INVERSION_INVERTED_A7          (0xA7)

void Initialize_CFAL12832DB(void)
  {
  //Thump the reset.  
  _delay_ms(1);
  CLR_RST;
  _delay_ms(1);
  SET_RST;
  _delay_ms(5);

  //Set the display to sleep mode for the rest of the init.
  sendcommand(SSD1306B_DISPLAY_OFF_YES_SLEEP_AE);

  //Set the clock speed, nominal ~105FPS
  //Low nibble is divide ratio
  //High level is oscillator frequency
  sendcommand(SSD1306B_CLOCK_DIVIDE_PREFIX_D5);
  sendcommand(0x80);//177Hz measured

  //Set the multiplex ratio to 1/32
  //Default is 0x3F (1/64 Duty), we need 0x1F (1/32 Duty)
  sendcommand(SSD1306B_MULTIPLEX_RATIO_PREFIX_A8);
  sendcommand(0x1F);  

  //Set the display offset to 0 (default)
  sendcommand(SSD1306B_DISPLAY_OFFSET_PREFIX_D3);
  sendcommand(0x00);

  //Set the display RAM display start line to 0 (default)
  //Bits 0-5 can be set to 0-63 with a bitwise or
  sendcommand(SSD1306B_DISPLAY_START_LINE_40);

  //Enable DC/DC converter, 7.5v
  sendcommand(SSD1306B_DCDC_CONFIG_PREFIX_8D);
  sendcommand(SSD1306B_DCDC_CONFIG_7p5v_14);

  //Map the columns correctly for our OLED glass layout
  sendcommand(SSD1306B_SEG0_IS_COL_127_A1);

  //Set COM output scan correctly for our OLED glass layout
  sendcommand(SSD1306B_SCAN_DIR_UP_C0);

  //Set COM pins correctly for our OLED glass layout
  sendcommand(SSD1306B_COM_CONFIG_PREFIX_DA);
  sendcommand(SSD1306B_COM_CONFIG_SEQUENTIAL_LEFT_02);

  //Set Contrast Control / SEG Output Current / Iref
  sendcommand(SSD1306B_CONTRAST_PREFIX_81);
  sendcommand(0xBF);  //magic # from factory

  //Set precharge (low nibble) / discharge (high nibble) timing
  //precharge = 1 clock
  //discharge = 15 clocks
  sendcommand(SSD1306B_PRECHARGE_PERIOD_PREFIX_D9); //Set Pre-Charge period        
  sendcommand(0xF1); 

  //Set VCOM Deselect Level
  sendcommand(SSD1306B_VCOMH_DESELECT_PREFIX_DB);
  sendcommand(SSD1306B_VCOMH_DESELECT_0p83xVCC_30); 

  //Make sure Entire Display On is disabled (default)
  sendcommand(SSD1306B_ENTIRE_DISPLAY_NORMAL_A4);

  //Make sure display is not inverted (default)
  sendcommand(SSD1306B_INVERSION_NORMAL_A6);

  //Get out of sleep mode, into normal operation
  sendcommand(SSD1306B_DISPLAY_ON_NO_SLEEP_AF);
  }
//============================================================================
uint8_t
  current_screen;
void setup( void )
  {
  //Set up port B
  DDRB = 0x2F;

  //Drive the ports to a reasonable starting state.
  SET_RST;
  SET_RS;
  SET_CS;
  CLR_MOSI;
  CLR_SCK;

  // Initialize SPI. By default the clock is 4MHz. The chip is good to 10 MHz
  SPI.begin();
  //Bump the clock to 8MHz. Appears to be the maximum.
  SPI.setClockDivider(SPI_CLOCK_DIV2);
  SPI.setDataMode(SPI_MODE0);
  SPI.setBitOrder(MSBFIRST);   

  //Fire up the I2C OLED
  Initialize_CFAL12832DB();

  current_screen=0;
  }
//============================================================================
void  loop(void)
  {
  //Put up the bitmap from flash.
  show_128_x_4_bitmap_faster((SCREEN_IMAGE *)pgm_read_word(&screens[current_screen]));
  //Wait a bit . . .
  _delay_ms(750);
  //Move to the next screen.  
  current_screen++;
  if(current_screen==NUMBER_OF_SCREENS)
    {
    current_screen=0;
    }
  }
//============================================================================

